.twitter-typeahead {
  width: 100%;
}

.tt-menu {
 background-color: #fff;
 padding: 8px 6px;
 border: 1px solid rgba(0,0,0,0.2);
 border-radius: 8px;
 box-shadow: 0 5px 10px rgba(0,0,0,0.2);
 color: black;
}

.tt-suggestion:hover {
 background-color: #f5f5f5 ;
 cursor:pointer;
}

.tt-input, .tt-hint {
  width: 250px;
}

/* overall page elements */
.page-container {
    margin-left: auto;
    margin-right: auto;
    padding-left: 25px;
    padding-right: 25px;
    width: 1045px;
}

/* navbar elements*/
.navbar-knowledge, .navbar-text, .glyphicon-navbar {
  color: #cacccd;
  background-color: #565a5c;
  border-radius: 0px;
}

.navbar-text[href]:hover {
  color: white;
}

#searchbar {
  border: 1px solid #ccc;
  margin-top:10px;
  width: 250px;
}

.panel {
  border: 1px solid #dce0e0;
  color: #565a5c;
  background-color: white;
  border-radius: 0;
  margin-bottom: 0px;
}

.panel-header {
    color: #565a5c;
    font-size: 16px;
    padding-top: 14px;
    padding-bottom: 14px;
    border-bottom: 1px solid #dce0e0;
}

.panel-body, .panel-header {
  padding-top: 20px;
  padding-bottom: 20px;
}

.panel-dark, .panel-header {
    background-color: #edefed;
}

.panel-header, .panel-body, ul.panel-body > li, ol.panel-body > li, .panel-footer {
    position: relative;
    margin: 0;
    padding: 20px;
    border-top: 1px solid #dce0e0;
}

.panel-light {
    background-color: white;
}

body {
  color: #565a5c;
  background-color: whitesmoke;
  font-size: 14px;
  font-family: 'Lato';
  /*background-color: #F8F8F8;*/
}

a {
  color: #00a699;
  text-decoration: none;
}

a:hover {
  color: #9CA299;
  opacity: 0.5;
  text-decoration: none;
}

ul {
    margin-top: 10px;
    margin-bottom: 10px;
}

h1 {
  font-size: 44px;
}
h2 {
  margin-top: 0;
  margin-bottom: 0;
  color: #484848;
  font-size: 32px;
  font-weight: bolder;
}
h3 {
  font-size: 24px;
}
h4 {
  font-size: 18px;
}
h5 {
  font-size: 16px;
}
h6 {
  font-size: 14px;
}

img {
    max-width:100%;
}

.logo {
  display: block;
  font-size: 21px;
  padding: 0 15px;
  margin-bottom: 10px;
  font-weight: 900;
  font-family: 'Playfair Display';
}

.glyphicon {
  vertical-align: baseline;
}

.link-reset {
  color:black;
}

.label-primary,
.label-default,
.label-subscribed,
.label-unsubscribed {
  font-size: 14px;
  border: 1px solid;
  border-radius: 15px;
  border-color: #9CA299;
  font-weight: normal;
  display: inline-block;
  margin-bottom: 4px;
}

.label-primary {
  color: white;
  background-color: #FF5A5F;
}

.label-default {
  color: #9CA299;
  background-color: transparent;
}

.label-tag[href]:hover {
  color: #00A699;
  border-color: #00A699;
  background-color: #fff;
  opacity: 1;
}

.label-subscribed {
  color: white;
  background-color: #00A699;
  opacity: 0.7;
}

.label-unsubscribed {
  color: #9CA299;
  background-color: inherit;
}

.btn-primary {
  color: #fff;
  background-color: #00A699;
  border: 1px solid #00A699;
}

.btn.btn-default.btn-subscribe,
.btn.btn-default.btn-subscribe:hover {
  background-color: #fff;
  border: 1px solid #00A699;
  color: #00A699;
}

.btn-unsubscribe,
.btn-unsubscribe:hover {
  background-color: #00A699;
  border: 1px solid #00A699;
  color: #fff;
}


.index-view-btn-group {
  border-color: #9CA299;
  margin-left: -13px;
}

.glyphicon-white {
  color: #FFFFFF
}

.btn-default.prev_btn,
.btn-default.next_btn {
  border: 1px solid  #ffb400;
  color:  #ffb400;
}

.btn-default {
  background-color: whitesmoke;
  border: 1px solid #9CA299;
  color:  #9CA299;
}

.glyphicon-filled, .glyphicon-clickable {
  color: #9CA299;
}

.feed-author {
  padding-bottom: 5px;
}

.feed-date {
  color: #565a5c;
  padding-bottom: 10px;
}

.feed-date, .feed-tldr {
  color: #565a5c;
}

.feed-post {
  border-bottom: 1px solid #F8F8F8;
}

.feed-post-counts {
  color: #aaa;
  margin-right: 3px;
  margin-left: 15px;
}

.content-offset {
  margin-left: 50px;
}

.content-offset-right {
  margin-right: 50px;
}

.js-example-tags {
  position:inherit !important;
  display:inline-block;
}

.tag--subscribed {
  color:white;
  background-color: #FF5A5F;
  border-color: transparent;
}

.btn-batch-tags-invisible {
  padding: 2px 0px;
  background-color: Transparent;
}

.btn-batch-tags-invisible > .glyphicon {
  color: #9CA299;
}

.batch-tags-text-field {
  width: 500px;
}

.label-primary[href]:hover,
.label-primary[href]:focus,
.label-primary[href]:active,
.btn-primary:hover,
.btn-primary:focus,
.btn-primary:active {
  color:white;
  background-color: #9CA299;
  border-color: #9CA299;
  opacity: 0.5;
}

.label-default[href]:focus,
.label-default[href]:active,
.btn-default:hover,
.btn-default:focus,
.btn-default:active {
  color: #9CA299;
  background-color: white;
  border-color:  #9CA299;
  opacity: 0.5;
}

.feed-post-counts, .glyphicon:first-child {
  margin-left: 0px;
}

.content {
  border-left: 1px solid #F8F8F8;
  background-color: white;
  line-height: 1.6em;
  color: #555;
  font-size: 18px;
}

#renderedMarkdown h1,
#renderedMarkdown h2,
#renderedMarkdown h3,
#renderedMarkdown h4,
#renderedMarkdown h5,
#renderedMarkdown h6 {
  margin-top: 50px;
  margin-bottom: 25px;
  font-weight: bold;
}

#renderedMarkdown h1 {
  margin-top: 0;
  font-weight: bold;
}

/*#renderedMarkdown p {
  margin: 0 0 25px;
}*/

#renderedMarkdown p,
#renderedMarkdown h1,
#renderedMarkdown h2,
#renderedMarkdown h3,
#renderedMarkdown .codehilite,
#renderedMarkdown ul {
  width: 80%;
  color: #484848;
}

#renderedMarkdown p > img {
  margin-top: 25px;
  margin-bottom: 25px;
  margin-left: auto;
  margin-right: auto;
}

#renderedMarkdown table {
  width: 80%;
  margin-top: 10px;
  margin-bottom: 10px;
  display: block;
  overflow-x: auto;
  border-spacing: 10px;
}

#renderedMarkdown th, #renderedMarkdown td {
  border: 1px solid grey;
  padding: 0.5em;
}

#renderedMarkdown .codehilite {
  margin-top: 25px;
  margin-bottom: 25px;
}

#renderedMarkdown ul > li > p {
  margin-left: 0;
}

.row-space-1 {
  margin-bottom: 6.25px; }

.row-space-top-1 {
  margin-top: 6.25px; }

.row-space-2 {
  margin-bottom: 12.5px; }

.row-space-top-2 {
  margin-top: 12.5px; }

.row-space-3 {
  margin-bottom: 18.75px; }

.row-space-top-3 {
  margin-top: 18.75px; }

.row-space-4 {
  margin-bottom: 25px; }

.row-space-top-4 {
  margin-top: 25px; }

.row-space-5 {
  margin-bottom: 31.25px; }

.row-space-top-5 {
  margin-top: 31.25px; }

.row-space-6 {
  margin-bottom: 37.5px; }

.row-space-top-6 {
  margin-top: 37.5px; }

.row-space-7 {
  margin-bottom: 43.75px; }

.row-space-top-7 {
  margin-top: 43.75px; }

.row-space-8 {
  margin-bottom: 50px; }

.row-space-top-8 {
  margin-top: 50px; }

.row-space-top-12 {
  margin-top: 75px; }

.tooltip.bottom .tooltip-inner {
  background-color:transparent;
  color:grey;
  }

.tooltip.bottom .tooltip-arrow {
  border-bottom-color: grey;
}

a.no-underline {
  text-decoration: none;
  color: #9CA299;
}

span.index-view-name {
  margin-left: 10px;
}

.btn-active, .btn-active > span {
  color: white;
  background-color: #00a699;
}

.btn-active:hover {
  background-color: #00a699;
  opacity: 0.8;
}

.glyphicon-active {
  color: white;
  opacity: 1;
}

.pagination-group > a {
  border: 0;
}

.page-btn {
  padding-bottom: 2px;
}

.page-btn > a:visited {
  background-color: inherit;
}

.pagination-group > a:hover,
.pagination-group > a:active,
.pagination-group > a:focus,
.pagination-group > a:active:focus,
.pagination-group > a:active:hover {
  background-color: inherit;
  border: 0px;
  opacity: inherit;
  outline: none;
  box-shadow: none;
  color: inherit;
}

.page-btn:hover, .page-btn:active {
  background-color: inherit;
  border: 0px;
  opacity: inherit;
  outline: none;
}

.active-page, .active-page:hover {
  color: #00a699;
  padding-bottom: 2px;
  background-color: inherit;
  border: 0px;
  opacity: inherit;
}

.feed-post-counts > .glyphicon-heart-empty,
.feed-post-counts > .glyphicon-comment {
  padding-left: 10px;
}

.cluster-header {
  background-color:white;
  margin-bottom:0px;
  padding-top: 15px;
  padding-bottom:15px;
  border-bottom: 1px lightgrey solid
}

.group-by-header > h5,
.sort-by-header > h5 {
  color: #9CA299;
  font-weight: 700;
}

.group-by-options {
  margin-left: -40px;
}

.sort-by-options {
  margin-left: -100px;
}
.radio-cluster {
  font-weight: 100;
  padding-top: 10px;
  color: #9ca299;
}

.radio-cluster > label {
  font-weight: 100;
}

.radio-cluster > label > input[type='radio'] {
  -webkit-appearance:none;
  width:15px;
  height:15px;
  border-radius:50%;
  outline:none;
  box-shadow:0 0 5px 0px #9CA299 inset;
}

.radio-cluster > label > input[type='radio']:hover {
  border-radius:50%;
  box-shadow:0 0 5px 0px #9CA299 inset;
}

.radio-cluster > label > input[type='radio']:before{
  content:'';
  display:block;
  width:75%;
  height:70%;
  margin: 15% auto;
  border-radius:50%;
}

.radio-cluster > label > input[type='radio']:checked:before {
  background:#9CA299;
}

#cluster_list {
  background-color: white;
  margin-top: 0px;
  margin-right: -15px;
  margin-left: -15px;
  padding-top: 10px
}

.cluster-dropdown {
  color: #9CA299;
  opacity: 0.6;
  font-size: 10px
}

.webeditor-form {
  padding-left: 15px;
}

.webeditor-form > label {
  font-weight: 200;
}

#post_author + span.select2-container {
  display: inherit;
}

.glyphicon-edit, .glyphicon-remove {
  color: #565A5C;
}

#post_comment_btn,
#btn_save:active,
#btn_save:focus,
#btn_save:hover,
#btn_in_review.btn-primary,
#btn_publish:active,
#btn_publish:focus,
#btn_publish:hover,
#btn_publish.btn-primary,
.btn-email-reviewers{
  background-color: #00a699;
  border: 1px solid #00a699;
  color: #fff;
}

.create-btn-group > a {
  background-color: #00A699;
  border: 1px solid #00A699;
}

.btn[data-target*='#show_posts_']:active,
.btn[data-target*='#show_posts_']:focus {
  outline: none;
  box-shadow: none;
  -webkit-box-shadow: none;
}

