{% extends "base.html" %}

{% block title %} Knowledge Repo 404 {% endblock %}

{% block content %}

<div class="col-12">
  <div class="panel">
    <center>
        <div class="panel-header">
          Error!
        </div>
    </center>
    
    <center>
        <div class="panel-body">
          <div class="row row-space-2"></div>
          <h3> Uh oh! Looks like something broke </h3>
          <text> Try clicking around on the other links while we try and get this page up and running </text>
        </div>
    </center>
  </div>
</div>

{% endblock %}
