{% extends "markdown-base.html" %}

{% block title %} Markdown {% endblock %}

{% block content %}
{{ super() }}
<div>
  <h1> {{ post_path }} </h1>
  <div style="white-space: pre-wrap;">
  {{ raw_post | safe }}
  </div>
</div>
{% endblock %}

{% block scripts %}
{{ super() }}
<script>
$(".btn-raw").addClass("btn-active")
</script>
{% endblock %}
